@@ -242,7 +242,7 @@ IOUSBMassStorageClass::BulkOnlySendCBWPacket(
    	STATUS_LOG(("%s: BulkOnlySendCBWPacket sent\n", getName() ));
 	status = GetBulkOutPipe()->Write( boRequestBlock->boPhaseDesc, &boRequestBlock->boCompletion );
    	STATUS_LOG(("%s: BulkOnlySendCBWPacket returned %d\n", 
-   				getName(), 
+   				getName(),
    				status));
 	return status;
 }
@@ -269,9 +269,7 @@ IOUSBMassStorageClass::BulkOnlyTransferData(
 					GetDataBuffer( boRequestBlock->request ), 
 					GetTimeoutDuration( boRequestBlock->request ),  // Use the client's timeout for both
 					GetTimeoutDuration( boRequestBlock->request ),
-#if 0
 					GetRequestedDataTransferCount( boRequestBlock->request ),
-#endif
 					&boRequestBlock->boCompletion );
 	}
 	else if (GetDataTransferDirection(boRequestBlock->request) == kSCSIDataTransfer_FromInitiatorToTarget)
@@ -280,9 +278,7 @@ IOUSBMassStorageClass::BulkOnlyTransferData(
 					GetDataBuffer( boRequestBlock->request ), 
 					GetTimeoutDuration( boRequestBlock->request ),  // Use the client's timeout for both
 					GetTimeoutDuration( boRequestBlock->request ),
-#if 0
 					GetRequestedDataTransferCount( boRequestBlock->request ),
-#endif
 					&boRequestBlock->boCompletion );
 	}
 
@@ -371,10 +367,10 @@ IOUSBMassStorageClass::BulkOnlyExecuteCommandCompletion(
 		case kBulkOnlyCommandSent:
 		{
    			STATUS_LOG(("%s: kBulkOnlyCommandSent returned %d\n", getName(), resultingStatus));
-
+			
 			// Release the memory descriptor for the CBW
 			boRequestBlock->boPhaseDesc->release();
-
+			
 			if(( resultingStatus == kIOReturnDeviceError )
 				|| ( resultingStatus == kIOReturnNotResponding ))
 			{
